require 'enumerator'

module Enumerable

  # Iterate through slices. If slicing +step+ is not
  # given, the the arity if the block is used.
  #
  #   x = []
  #   [1,2,3,4].each_by{ |a,b| x << [a,b] }
  #   x  #=> [ [1,2], [3,4] ]
  #
  #   x = []
  #   [1,2,3,4,5,6].each_by(3){ |a| x << a }
  #   x  #=> [ [1,2,3], [4,5,6] ]

  def each_by(step=nil, &yld)
    if step
      each_slice(step,&yld)
    else
      step = yld.arity.abs
      each_slice(step,&yld)
    end
  end

end


#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#
=begin test

  require 'test/unit'

  class TCEnumerable < Test::Unit::TestCase

    def test_01
      x = []
      [1,2,3,4].each_by{ |a,b| x << [a,b] }
      o = [[1,2],[3,4]]
      assert_equal( o, x )
    end

    def test_02
      x = []
      [1,2,3,4,5].each_by{ |a,b,c| x << [a,b,c] }
      o = [[1,2,3],[4,5,nil]]
      assert_equal( o, x )
    end

    def test_03
      x = []
      [1,2,3,4].each_by(2){ |a,b| x << [a,b] }
      o = [[1,2],[3,4]]
      assert_equal( o, x )
    end

    def test_04
      x = []
      [1,2,3,4,5,6,7,8].each_by(4){ |*a| x << a }
      o = [ [[1,2,3,4]],[[5,6,7,8]] ]
      assert_equal( o, x )
    end

    def test_05
      x = []
      [1,2,3,4,5,6].each_by(3){ |*a| x << a }
      o = [ [[1,2,3]],[[4,5,6]] ]
      assert_equal( o, x )
    end

    def test_06
      a = [1,2,3,4,5,6]
      r = []
      a.each_by(2){ |x,y| r << [x,y] }
      assert_equal( [[1,2],[3,4],[5,6]], r )
    end

    def test_07
      a = [1,2,3,4,5,6]
      r = []
      a.each_by(3){ |*e| r << e }
      assert_equal( [ [[1,2,3]], [[4,5,6]] ], r )
    end

  end

=end
